--[[---------------------------------------------------------------------------
    File:   HealBot-ReadyCheckFixer.lua
    Desc:   This file contains the core implementation for this addon.
-----------------------------------------------------------------------------]]

local ADDON_NAME = ...
local ADDON_VER = GetAddOnMetadata(ADDON_NAME, "Version"):match("^([%d.]+)")

--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
--[[                       Variables                                         ]]
--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

local EventFrame = nil
local ReadyCheckCountdownTimer = nil

local SOUND_CHANNEL = "Master" -- Comment this line out to disable completion sounds.
local SOUND_ALL_READY = 10157  -- (10157=A_HELL_Bldgrd_Ready01, 37656=ui-scenario-bonusobjective-success, 9632=bloodelffemalevocalcheer, 77355=clientscene-70-artif-pr-oh-velensvision-lightstart, 44322=ui-garrison-commandtable-100success, 73281=ui-orderhall-talent-ready-check, 127043=vo-pckultiranhumanfemale-openfire, 17316=LFG_Rewards)
local SOUND_TIMED_OUT = 67825  -- (67825=vo-humanfemale-oops, 34039=UI_FightClub_Defeat)

--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
--[[                            Functions                                    ]]
--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

----print("Load Time:", GetTime())  -- For debugging.

-------------------------------------------------------------------------------
local function initialize()
    ----print("Initialize Time:", GetTime())  -- For debugging.
    if not IsAddOnLoaded("HealBot") then
        print("|c00ff0000".."WARNING: "..ADDON_NAME.." was NOT loaded because it requires HealBot.|r") -- Color format = xRGB.
        return  -- Do not load this addon.  (It requires HealBot.)
    ----elseif (HealBot_ReadyCheck_Start and HealBot_ReadyCheck_Confirm and HealBot_ReadyCheck_Finish) then
    ----    print("|c00ff0000".."WARNING: "..ADDON_NAME.." was NOT loaded because HealBot has fixed readychecks itself.|r") -- Color format = xRGB.
    ----    return  -- Do not load this addon.  (HealBot has fixed readychecks itself.)
    end

    -- If we get here, it is safe to load this addon.
    print("|c007f7fff"..ADDON_NAME.." "..ADDON_VER.." loaded.|r") -- Color format = xRGB.
    EventFrame = CreateFrame("Frame")

    EventFrame:RegisterEvent("READY_CHECK")
    EventFrame:RegisterEvent("READY_CHECK_CONFIRM")
    EventFrame:RegisterEvent("READY_CHECK_FINISHED")

    EventFrame:SetScript("OnEvent", function(self, event, ...)
        if (event == "READY_CHECK") then
            HealBot_ReadyCheck_Start_Fixer(...)
        elseif (event == "READY_CHECK_CONFIRM") then
            HealBot_ReadyCheck_Confirm_Fixer(...)
        elseif (event == "READY_CHECK_FINISHED") then
            HealBot_ReadyCheck_Finish_Fixer()
        end
    end)

    EventFrame:SetScript("OnUpdate", function(self, elapsedSeconds)
        if ReadyCheckCountdownTimer then
            ReadyCheckCountdownTimer = ReadyCheckCountdownTimer - elapsedSeconds
            if (ReadyCheckCountdownTimer <= 0) then
                ReadyCheckCountdownTimer = nil
                
                -- Clear all readycheck icons.
                for xUnit, xButton in pairs(HealBot_Unit_Button) do
                    if xButton.icon.debuff.readycheck then
                        xButton.icon.debuff.readycheck = false
                        xButton.aura.debuff.check = true  -- Forces immediate update.
                    end
                end
            end
        end
    end)
end

-------------------------------------------------------------------------------
function HealBot_ReadyCheck_Start_Fixer(initiatorFullName, maxWaitSecs)
    if not initiatorFullName then initiatorFullName=UnitName("player") end  -- Only for testing.
    
    local iniPlayer, iniRealm = string.split("-", (initiatorFullName or ""))
    if not iniRealm then iniRealm = GetRealmName() end

    ReadyCheckCountdownTimer = nil
    for xUnit, xButton in pairs(HealBot_Unit_Button) do
        local xPlayer, xRealm = UnitFullName(xUnit)
        if (xPlayer == iniPlayer and xRealm == iniRealm) then 
            xButton.icon.debuff.readycheck = READY_CHECK_READY_TEXTURE  -- Mark initiator as READY.
        else 
            xButton.icon.debuff.readycheck = READY_CHECK_WAITING_TEXTURE 
        end
        xButton.aura.debuff.check = true  -- Forces immediate update.
    end
end

-------------------------------------------------------------------------------
function HealBot_ReadyCheck_Confirm_Fixer(unit, ready)
    local xButton = HealBot_Unit_Button[unit]
    if not xButton then return end
    
    local prev = xButton.icon.debuff.readycheck
    if (ready == 1 or ready == true) then
        xButton.icon.debuff.readycheck = READY_CHECK_READY_TEXTURE
    else
        xButton.icon.debuff.readycheck = READY_CHECK_NOT_READY_TEXTURE
    end
    
    if (xButton.icon.debuff.readycheck ~= prev) then
        xButton.aura.debuff.check = true  -- Forces immediate update.
    end
end

-------------------------------------------------------------------------------
function HealBot_ReadyCheck_Finish_Fixer()
    local timeNow = GetTime()
    local notReadyCnt = 0
    for xUnit, xButton in pairs(HealBot_Unit_Button) do
        if (xButton.icon.debuff.readycheck ~= READY_CHECK_READY_TEXTURE) then
            notReadyCnt = notReadyCnt + 1
        end
        if (xButton.icon.debuff.readycheck == READY_CHECK_WAITING_TEXTURE) then
            xButton.icon.debuff.readycheck = READY_CHECK_AFK_TEXTURE  -- Flag players that are AFK.
            xButton.aura.debuff.check = true  -- Forces immediate update.
        end
    end
    
    -- Set the time to clear all readycheck icons.
    if not ReadyCheckCountdownTimer then 
        ReadyCheckCountdownTimer = DEFAULT_READY_CHECK_STAY_TIME 
    end
    
    -- Print summary in chat, and play a sound.
    if not InCombatLockdown() then
        local msg = "* ReadyCheck "
        if (notReadyCnt == 0) then
            msg = msg.."SUCCEED  (Everyone is ready.)"
            if SOUND_CHANNEL then PlaySound(SOUND_ALL_READY, SOUND_CHANNEL) end
        else
            return
            ----msg = msg.."FAILED"  ----"FAILED  ("..notReadyCnt.." not ready.)"
            ----if SOUND_CHANNEL then PlaySound(SOUND_TIMED_OUT, SOUND_CHANNEL) end
        end
        local info = ChatTypeInfo["SYSTEM"]
        DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
    end
end

--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
--[[                            INITIALIZE ADDON                             ]]
--:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

C_Timer.After(5, initialize) -- Delay initializing this addon for a few seconds 
                             -- to be sure HealBot gets loaded first.
--- End of File ---